/** This component provides an interface to 2D javascript arrays. */


/*************** COMPONENT *************************/

function SimpleFile(file) {
  this.file = file;
  if (!file.exists()) {
    file.create( Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420 );
  }
}
SimpleFile.prototype = {
  /** Print the given string to the file, followed by a newline.
   * @param str data to write
   * @return none
   */
  println: function (str) {
    // create write stream on demand
    if (!this.fos) {
      this.fos = Components.classes["@mozilla.org/network/file-output-stream;1"]
      .createInstance( Components.interfaces.nsIFileOutputStream );
      this.fos.init( this.file, 0x04 | 0x08 | 0x20, 420, 0 );
    }

    str += "\n";
    this.fos.write(str, str.length);
  },

  /** Read one line from the file.
   * @return data read from stream including newline, or null on error
   */
  readln: function() {

    //dump("CALL SimpleFile::readln()");

    // create read stream on demand
    if (!this.fis) {
      //dump("create input stream component");
      this.fis = Components.classes["@mozilla.org/network/file-input-stream;1"].
	createInstance( Components.interfaces.nsIFileInputStream );

      //dump("init file input stream");
      this.fis.init( this.file, 0x01, 00004, null );

      this.lis = this.fis.QueryInterface( Components.interfaces.nsILineInputStream );
    }

    // create object for use as xpidl inout param
    var line = { value: null };
    var status = this.lis.readLine(line);
    return line.value;
  },

  /** Close all open streams (i.e., read and write) used by file.
   */
  close: function() {
    if (this.fis)
      this.fis.close();
    if (this.fos)
      this.fos.close();
  },
}


function mtDataSet() {
  // allow direct access through javascript (so [data] member can be set)
  this.wrappedJSObject = this;
}

mtDataSet.prototype = {

  /** JavaScript specific.
   * Initialize javascript object with given 2D array. 
   * @param data 2d array, first dimension MUST be > 0
   */
  initFromArray: function(data) {
    this.data = data;
    this.n_cols = data[0].length;
  },

  idx: 0,
  n_cols: 0,


  //////// mtIDataSet Functions ////////////

  getNumRows: function() {
    return this.data.length;
  },
  getNumCols: function() {
    return this.n_cols;
  },

  getColumn: function (idx, count, values) {
    values.value = Array();
    count.value = this.n_cols;

    if (idx >= this.data.length || idx < 0)
      return Components.results.NS_ERROR;

    var arr = this.data[ idx ];
    for (var el in arr)
      values.value.push(el);

    return Components.results.NS_OK;
  },

  getNext: function (count, values) {
    var arr, el;

    if (this.idx >= this.data.length ||
	count != this.n_cols)
      return Components.results.NS_ERROR;

    arr = this.data[ this.idx++ ];
    for (el in arr)
      values.push(el);
    return Components.results.NS_OK;
  },

  hasMoreElements: function () {
    if (this.idx < this.data.length)
      return true;
    return false;
  },

  writeToFile: function (file) {
    var f, row;
    f = new SimpleFile(file);
    f.println(this.getNumRows() + " " + this.getNumCols());
    for (var i=0; i < this.data.length; ++i)
      f.println( this.data[i].join(" ") );
    f.close();

    return Components.results.NS_OK;
  },

  readFromFile: function (file) {
    var f, str;
    f = new SimpleFile(file);

    str = f.readln();
    if (str == null)
      return Components.results.NS_ERROR;

    str = str.split(" ");
    var rows = str[0];
    var cols = str[1];

    // copy number of columns to object property
    this.n_cols = cols;

    // Create data array
    var data = Array();
    for (var i=0; i < rows; ++i) {
      str = f.readln();
      if (str == null)
	return Components.results.NS_ERROR;

      data.push( str.split(" ") );
    }

    this.data = data;

    return Components.results.NS_OK;
  },

  
  //////// Component Housekeeping //////////

  get value()       { return this.val; },
  set value(newval) { return this.val = newval; },
  QueryInterface : function (iid) {
    if (iid.equals(Components.interfaces.mtIDataSet) ||
	iid.equals(Components.interfaces.nsISupports))
      return this;

    Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
    return null;
  },

}


/*************** MODULE *************************/

  var myModule = {
    registerSelf: function mod_regself(compMgr, fileSpec, location, type) {
      var compReg = compMgr.QueryInterface( Components.interfaces.nsIComponentRegistrar );
      compReg.registerFactoryLocation(this.myCID, 
				      "mtDataSet", 
				      this.myProgID,
				      fileSpec, location, type );
    },

    getClassObject: function (compMgr, cid, iid) {
      if (!cid.equals(this.myCID))
	throw Components.results.NS_ERROR_NO_INTERFACE;

      if (!iid.equals(Components.interfaces.nsIFactory))
	throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

      return this.myFactory;
    },


    myCID:    Components.ID('{38E80350-3961-4feb-9BA4-A1F4F8C34554}'),
    myProgID: '@pdx.edu/moztorch/mtDataSet;1',


    /* Factory Object */
    myFactory : {
      /* Construct instance of specified interface. */
      createInstance: function (outer, iid) {
	if (outer != null)
	  throw Components.results.NS_ERROR_NO_AGGREGATION;

	return (new mtDataSet()).QueryInterface(iid);
      }
    },

    canUnload : function(compMgr) {
      return true;
    }

  };


// NSGetModule: Return the nsIModule object.
function NSGetModule(compMgr, fileSpec) {
  return myModule;
}




function assert(fact) {
  if (!fact) {
    dump("Assert Failed: ...");
  }
}

function dump(s) {
  var consoleService = Components.classes['@mozilla.org/consoleservice;1']
    .getService(Components.interfaces.nsIConsoleService);
  consoleService.logStringMessage("[mtDataSet] " + s);
}
  


